/*
	desc      : libvlc wxWidigets test application
	author    : raphael smart
	date      : october 2012

	licence/disclaimer:

	This software is provided 'as-is', without any express or implied warranty.
	In no event will the author be held liable for any damages arising from
	the use of this software.
*/
namespace wxvlc
constant __MODULENAME__ = "wxvlc"

include std/os.e
include std/filesys.e
include std/cmdline.e
include std/map.e
include std/console.e
include std/text.e     as txt

include include/libvlc.e as vlc

include test/wxeud.e as wx

enum 

	false = 0,
	true

atom 

	mainFrame     ,
	mainPanel     ,
	btnPlay       ,
	btnPause      ,
	btnMute       ,
	btnFullscreen ,
	btnClose      ,
	btnFile       ,
	fileDialog    ,
	mediaTimer

atom vlc = 0, media = 0, mediaplayer = 0

constant ONE_SECOND = 1000

procedure loadMedia(sequence fileName)

	if length(fileName) then

		vlc:assert(file_exists(fileName), "file '" & fileName & "' does not exist")

		--// create a new item

		media = vlc:mediaNewPath(vlc, fileName)

		vlc:assert(media > 0, "could not initialise vlc media for media file '" & fileName & "'")

		--// create a media player playing environment

		if mediaplayer > 0 then
			vlc:mediaPlayerStop(mediaplayer)
			vlc:mediaPlayerRelease(mediaplayer)
		end if

		mediaplayer = vlc:mediaPlayerNewFromMedia(media)

		vlc:assert(mediaplayer > 0, "could not initialise vlc media player")

		vlc:mediaRelease(media)

		media = 0

		--// set media player capabilities

		vlc:mediaPlayerSetPause(mediaplayer, true)

		--// start playback

		vlc:mediaPlayerPlay(mediaplayer)

	end if

end procedure

procedure onFile(atom this, atom eventType, atom id, atom event)
	if wx:show_modal(fileDialog) then
		loadMedia(wx:get_fd_path(fileDialog))
	end if
end procedure

procedure onPlay(atom this, atom eventType, atom id, atom event)
	vlc:mediaPlayerPlay(mediaplayer)
end procedure

procedure onPause(atom this, atom eventType, atom id, atom event)
	vlc:mediaPlayerPause(mediaplayer)
end procedure

procedure onMute(atom this, atom eventType, atom id, atom event)
	vlc:audioToggleMute(mediaplayer)
end procedure

procedure onFullscreen(atom this, atom eventType, atom id, atom event)
	vlc:toggleFullscreen(mediaplayer)
end procedure

procedure onClose(atom this, atom eventType, atom id, atom event)
	if mediaplayer > 0 then
		vlc:mediaPlayerStop(mediaplayer)
		vlc:mediaPlayerRelease(mediaplayer)
	end if
	vlc:release(vlc)
	wx:exit_main()
end procedure

function formatTime(atom msecs)
	integer hours = floor(msecs / (3600 * ONE_SECOND))
	msecs -= hours * (3600 * ONE_SECOND)
	integer mins = floor(msecs / (60 * ONE_SECOND))
	msecs -= mins * (60 * ONE_SECOND)
	integer secs = floor(msecs / ONE_SECOND)
	return sprintf("%02d:%02d:%02d", { hours, mins, secs })
end function

procedure onTimer(atom this, atom eventType, atom id, atom event)
	wx:refresh_window(mainPanel)
end procedure

procedure onPaint(atom this, atom eventType, atom id, atom event)
	sequence info = ""
	atom elapsed = vlc:mediaPlayerGetTime(mediaplayer)
	if elapsed != -1 then
		info = "Elapsed : " & formatTime(elapsed)
	end if
	wx:wx_puts({ this, 120, 120 }, info)
end procedure

procedure initControls()

	mainFrame     = wx:create(wxFrame, { 0, -1, "wxWidgets VLC Test" })
	mainPanel     = wx:create(wxPanel, mainFrame)
	btnFile       = wx:create(wxButton, { mainPanel, -1, "Open File" })
	fileDialog    = wx:create(wxFileDialog, { mainPanel, "Select Media File", "", "", "*.*" })
	btnPlay       = wx:create(wxButton, { mainPanel, -1, "Play" })
	btnPause      = wx:create(wxButton, { mainPanel, -1, "Pause" })
	btnMute       = wx:create(wxButton, { mainPanel, -1, "Mute" })
	btnFullscreen = wx:create(wxButton, { mainPanel, -1, "Fullscreen" })
	btnClose      = wx:create(wxButton, { mainPanel, -1, "Exit" })
	mediaTimer    = wx:create(wxTimer,  { mainFrame, -1 })

	wx:start_timer(mediaTimer, ONE_SECOND, 0)
	
	atom horizSizer = wx:create(wxBoxSizer, wxVERTICAL)

	wx:add_window_to_sizer(horizSizer, btnFile, 0, wxTOP, 10)	
	wx:add_window_to_sizer(horizSizer, btnPlay, 0, wxTOP, 10)	
	wx:add_window_to_sizer(horizSizer, btnPause, 0, wxTOP, 10)	
	wx:add_window_to_sizer(horizSizer, btnMute, 0, wxTOP, 10)	
	wx:add_window_to_sizer(horizSizer, btnFullscreen, 0, wxTOP, 10)	
	wx:add_window_to_sizer(horizSizer, btnClose, 0, wxTOP, 10)	

	wx:set_sizer(mainPanel, horizSizer)

	wx:set_event_handler(btnFile,  -1, wxEVT_COMMAND_BUTTON_CLICKED, routine_id("onFile"))
	wx:set_event_handler(btnPlay,  -1, wxEVT_COMMAND_BUTTON_CLICKED, routine_id("onPlay"))
	wx:set_event_handler(btnPause, -1, wxEVT_COMMAND_BUTTON_CLICKED, routine_id("onPause"))
	wx:set_event_handler(btnMute,  -1, wxEVT_COMMAND_BUTTON_CLICKED, routine_id("onMute"))
	wx:set_event_handler(btnFullscreen, -1, wxEVT_COMMAND_BUTTON_CLICKED, routine_id("onFullscreen"))
	wx:set_event_handler(btnClose, -1, wxEVT_COMMAND_BUTTON_CLICKED, routine_id("onClose"))
	wx:set_event_handler(mainFrame,-1, wxEVT_TIMER, routine_id("onTimer"))
	wx:set_event_handler(mainPanel, wx:get_id(mainPanel) * 0 - 1, wxEVT_PAINT, routine_id( "onPaint" ))

end procedure

	--// main

	display("version   = [1]", { vlc:version() })
	display("compiler  = [1]", { vlc:compiler() })
	display("changeset = [1]", { vlc:changeset() })
	display("VLC_PLUGIN_PATH = [1]", { getenv("VLC_PLUGIN_PATH") })

	--// load vlc engine

	sequence vlcargv =
	{
		"-I", 
		"dummy"
	}
	
	vlc = vlc:new(vlcargv)

	--// init wx 

	initControls()

	--// run wx

	wx:wxMain(mainFrame)
